include "acts_global_values.gs"
include "acts_tools_script.gs"

//---------------------------------------------------------------------------
// aCTS Bridgecrane V0.1 Central Script - acts_bridgecrane_globals.gs
//-----------------------------------------------------------------------------
//   acts_bridgecrane           <-- acts_bridgecrane_script
//   acts_bridgecrane_script    <-- acts_bridgecrane_register
//   acts_bridgecrane_register  <-- acts_bridgecrane_reachable
//   acts_bridgecrane_reachable <-- acts_bridgecrane_moving
//   acts_bridgecrane_moving    <-- acts_bridgecrane_globals
// X acts_bridgecrane_globals   <-- acts_global_values
//   acts_global_values         <-- MapObject
//-----------------------------------------------------------------------------

class acts_bridgecrane_globals isclass acts_global_values
{
  //---------------------------------------------------------------------------
  // AssetInstanceSoup {
  // X ObjectDataSoup { ... },    Only container, contstack, contcrane
  // X LocationSoup { ... },      Only container, contstack, contcrane
  // X OrientationSoup { ... },   Only container, contstack, contcrane
  //   SkinDataSoup { ... },      Only container
  //   ContentDataSoup { ... },   Only container
  //   DispatcherDataSoup { ... } Only dispatcher
  //   ContainerSoups  { ... }    Only dispatcher
  //   ContstackSoups  { ... }    Only dispatcher
  //   ContcraneSoups  { ... }    Only dispatcher
  // }
  //---------------------------------------------------------------------------
  // ObjectDataSoup { goid GameObjectID, type string,
  //                  length string, height string,  
  //                  stack goid, place int,           Only container
  //                  active bool, action string },
  // activ: true means the object sended a register message while session run
  //        All objects were set false while initialising dispatcher and if
  //        they stay false their entries will be deleted next initialising.
  // actions: ["APPENDED", "INSERTED", "UPDATED"];
  //---------------------------------------------------------------------------
  Soup ObjectDataSoup = Constructors.NewSoup();
  //---------------------------------------------------------------------------
  // LocationSoup { bbx int, bby int,               // WorlCoordinate parameter
  //                px float, py float, pz float }
  //-------------------------------------------------------------------------
  Soup LocationSoup = Constructors.NewSoup();
  //-------------------------------------------------------------------------
  // OrientationSoup { rx float, ry float, rz float,    // in rad 
  //                   gx float, gy float, gz float }   // in 1/10°
  //-------------------------------------------------------------------------
  Soup OrientationSoup = Constructors.NewSoup();
  //-------------------------------------------------------------------------
  GameObjectID objectgoid = me.GetGameObjectID();
  string       objecttype = "CONTCRANE";
  Library      dispobj;
  GameObjectID dispgoid;
  //---------------------------------------------------------------------------
  // Reach area values of the bridge crane will be loaded from this assets
  // config.txt file 
  //---------------------------------------------------------------------------
  int cranebase_reacharea_x;      //  30000 in [mm]
  int cranebase_reacharea_y;      // 100000 in [mm]
  int cranebase_reacharea_z;      //  26000 in [mm]
  int cranebase_origin_inset_x;   //  15000 in [mm]
  int cranebase_origin_inset_y;   //  50000 in [mm]
  int cranebase_origin_inset_z;   //   5000 in [mm]
  //---------------------------------------------------------------------------
  int lift_zero_length = 1000;   // 1000 in [mm] mandatory length for animation
  //---------------------------------------------------------------------------
  int trolley_bottom_height;   // in [mm]
  int lift_bottom_height;      // in [mm] with lift z width as 1,000 [mm]
  int rotor_bottom_height;     // in [mm] with rotor z width as 500 [mm]
  int gear_bottom_height;      // in [mm] with gear z width as 500 [mm]
  //---------------------------------------------------------------------------
  // TBD: And not proofed yet. Differntiate between maximum container count
  //      for high cube and standard cube containers.
  //---------------------------------------------------------------------------
  // For now the maximum count is calculated from reacharea z dimension minus
  // one hc height to ensure the movement of containers by the crane. Because
  // of possibly different base heights of the stacks inside the crane's
  // reacharea, there are some more different (possibly smaller) maximum
  // stackheights.
  // --------------------------------------------------------------------------
  int max_container_count;     // Construction based max stackable containers
  // --------------------------------------------------------------------------
};

//-----------------------------------------------------------------------------
/* ****************************************************************************
   This is the scripted bridge crane object for an active container terminal
   system (aCTS). It is a kind of prototype of a bridge crane to let
   constructors know how the things are fitting and working for aCTS.
   ----------------------------------------------------------------------------
   Some hints here too (see config description too):
   * The script logs dependin on the acts_debug variable (See
     acts.global_values.gs). So the script contains some overhead for testing
     informations. 
   * Animations from Blender fbx runs from frame 1 ... n in Blender and fbx
     file. The import to Trainz will change this to frame 0 ... n-1, what is
     essential to know for precise transformations (moving and rotating) per
     animation frame.
   * The logical mesh hierarchy is as follows: cranebase-->
     -->cranebridge(py)-->trolley(px)-->lift(sz,pz)-->rotor(rz)-->gear(sx)
   * The cranes parts are imported to Trainz as own (animated) mesh objects
     with its respective origins at zero and its parented attachment points.
     All script calculations are based on this origin and the gaps to the
     attachment points for attaching the child meshes. The assets orientation
     is zero for x and y orientation.
   * The parent child hierarchy is realised within the mesh-table container.
   * The extensions values are in mm and define the reachable area of the
     cranes container gear. The dimensions x, y, z describe the cuboid
     length's, the gear may move and the insets x, y, z describes the location
     of the cranes origin within there.
   * The animations minimum frame 0 (resp. 1 in Blender) represents the locale
     relative minimal position (p = -inset) and the maximum frame (n = 
     dimension) the maximal position p=dimension-inset. Example: 
       dimension 80
       inset     30
       Trainz frames    0  10  20 30  40  50  60  70  80
       Blender frames   1  11  21 31  41  51  61  71  81
       Position       -30 -20 -10  0  10  20  30  40  50
   * The crane object (as well as the other object types) adjusts location to
     even mm and the orientation to 0.1 decimal degrees.
   * The movements of the cranes sub meshes will be realised with animation
     frame methods *MeshAnimation* from the MeshObject class in mashobject.gs
   ----------------------------------------------------------------------------
   Technical parameters for calculating script values 1 [mm] and 0.1 [°]
   ----------------------------------------------------------------------------
   Bridge  [BPY: 100,001F:    140m] BF[1 .. 100001] [0mm(1)100,000mm]
   Trolley [TPX:  30,001F:     30m] BF[1 ..  30001] [-15,000mm(1)+15,000mm]
   Lift    [LSZ:  25,001F:    1-26] BF[1 ..  25001] [1.000(0.001)26.000]
           [LPZ:  25,001F:     25m] BF[1 ..  25001] [+21,000mm(1)-5,000mm]
   Rotor   [ROZ:   1,801F:    180°] BF[1 ..   1801] [-90.0°(0.1)+90.0°]
   Gear    [GSY:  15,155F: 16.192m] BF[1 ..  16155] [1.000(0.001)16.154]
   ----------------------------------------------------------------------------
   Containertype                      10ft    20ft     40ft     45ft     53ft
   Gear scale scale/length  1.000m  2.991m  6.058m  12.192m  13.716m  16.154m
   ----------------------------------------------------------------------------
   More informations are documented within the config.txt and script comments.
   ----------------------------------------------------------------------------
   **************************************************************************** */
