include "acts_bridgecrane_globals.gs"
include "acts_tools_script.gs"
  
//-----------------------------------------------------------------------------
// aCTS Bridgecrane V0.1 Central Script - acts_bridgecrane_reachable.gs
//-----------------------------------------------------------------------------
//   acts_bridgecrane           <-- acts_bridgecrane_script
//   acts_bridgecrane_script    <-- acts_bridgecrane_register
//   acts_bridgecrane_register  <-- acts_bridgecrane_reachable
// X acts_bridgecrane_reachable <-- acts_bridgecrane_moving
// X acts_bridgecrane_moving    <-- acts_bridgecrane_globals
//   acts_bridgecrane_globals   <-- acts_global_values
//   acts_global_values         <-- MapObject
//-----------------------------------------------------------------------------

class acts_bridgecrane_moving isclass acts_bridgecrane_globals
{
  //---------------------------------------------------------------------------
  // Set zeroframe translations for the asset parts
  //---------------------------------------------------------------------------
  int bridge_zeroframe  = cranebase_origin_inset_y;
  int trolley_zeroframe = cranebase_origin_inset_x;
  int lift_zeroframe  = 0;     // Scale 1.000..25.000 (cranebase_reacharea_z)
  int rotor_zeroframe = 900;   // Decimal degree -90.0..+90.0 (precision 0.1°)
  int gear_zeroframe  = 2991;  // Scale 1.000-2.991-6.058-12.192-13.716-16.154
  //---------------------------------------------------------------------------
  void moveCranePartsToZeroframe(void)
  {
    me.SetMeshAnimationFrame("bridge-lod0",  bridge_zeroframe);
    me.SetMeshAnimationFrame("trolley-lod0", trolley_zeroframe);
    me.SetMeshAnimationFrame("lift-lod0",    lift_zeroframe);
    me.SetMeshAnimationFrame("rotor-lod0",   rotor_zeroframe);
    me.SetMeshAnimationFrame("gear-lod0",    gear_zeroframe);
  }
  //---------------------------------------------------------------------------
  // TBD: Where and when setting the fitting containers skin?
  //      Asset skins = GetAsset().FindAsset("skinskuid");
  //      SetFXTextureReplacement("skincolor40ftsc",skins,8);
  //      SetMeshVisible("cont40ftsc-lod0", true);
  //      me.SetMeshAnimationFrame("gear-lod0", 12191.0, 3.0);
  //---------------------------------------------------------------------------
};

/******************************************************************************
  -----------------------------------------------------------------------------
  In conjunction with the rotation of the cranearea there ar four cases to
  know about if a point lies within the reacharea, bounding edges are assumed
  to be outside.
  
   A B    B    
   D C   A C     0° ≤ w < 90°   CD < py < AB and AD < px < BC
          D
  
   B C    C
   A D   B D   90° ≤ w < 180°   AD < py < BC and AB < px < CD
          A
  
   C D    D 
   B A   C A  180° ≤ w < 270°   AB < py < CD and BC < px < AD   
          B

   D A    A    
   C B   D B  270° ≤ w < 360°   BC < py < AD and CD < px < AB
          C
  -----------------------------------------------------------------------------
*******************************************************************************/
class acts_bridgecrane_reachable isclass acts_bridgecrane_moving
{
  // --------------------------------------------------------------------------
  // Loading values from config.txt extensions-219486 cranedata soup
  // --------------------------------------------------------------------------
  void LoadCraneDataInfos(Soup sp)
  {
    cranebase_reacharea_x          
      = sp.GetNamedTagAsInt("cranebase_reacharea_x",0);
    cranebase_reacharea_y          
      = sp.GetNamedTagAsInt("cranebase_reacharea_y",0);
    cranebase_reacharea_z          
      = sp.GetNamedTagAsInt("cranebase_reacharea_z",0);
    cranebase_origin_inset_x              
      = sp.GetNamedTagAsInt("cranebase_inset_x",0);
    cranebase_origin_inset_y              
      = sp.GetNamedTagAsInt("cranebase_inset_y",0);
    cranebase_origin_inset_z              
      = sp.GetNamedTagAsInt("cranebase_inset_z",0);
    trolley_bottom_height   
      = sp.GetNamedTagAsInt("trolley_bottom_height",0);
    lift_bottom_height 
      = sp.GetNamedTagAsInt("lift_bottom_height",0);
    gear_bottom_height 
      = sp.GetNamedTagAsInt("gear_bottom_height",0);
    max_container_count
      = sp.GetNamedTagAsInt("max_container_count",0);
  }
  // --------------------------------------------------------------------------
  // Returns the vertices of the reacharea rectangle. The insets and dimensions
  // where deliverd over a objects config extension subsoup. Rotations x and y 
  // are zero.
  //  A---+-----B
  //  |   ix    |   ix insetx, iy insety
  //  |iy-O-----+   dx=AD=BC dimensionx, dy=AB=CD dimensiony
  //  |   |     |   RC[0..3]<=>ABCD 
  //  D---+-----C
  //---------------------------------------------------------------------------
  public WorldCoordinate[] GetReachareaCorners()
  {
    WorldCoordinate[] RC = new WorldCoordinate[4];    // Reacharea corners ABCD
    WorldCoordinate OR = me.GetMapObjectPosition();   // Origin of crane OR
    //-------------------------------------------------------------------------
    RC[0].baseboardX = OR.baseboardX;
    RC[0].baseboardY = OR.baseboardY;
    RC[0].x = OR.x - cranebase_origin_inset_x;
    RC[0].y = OR.y - cranebase_origin_inset_y;
    RC[0].z = OR.z;
    //-------------------------------------------------------------------------
    RC[1].baseboardX = OR.baseboardX;
    RC[1].baseboardY = OR.baseboardY;
    RC[1].x = OR.x - cranebase_origin_inset_x;
    RC[1].y = OR.y + cranebase_reacharea_y - cranebase_origin_inset_y;
    RC[1].z = OR.z;
    //-------------------------------------------------------------------------
    RC[2].baseboardX = OR.baseboardX;
    RC[2].baseboardY = OR.baseboardY;
    RC[2].x = OR.x + cranebase_reacharea_x - cranebase_origin_inset_x;
    RC[2].y = OR.y + cranebase_reacharea_y - cranebase_origin_inset_y;
    RC[2].z = OR.z;
    //-------------------------------------------------------------------------
    RC[3].baseboardX = OR.baseboardX;
    RC[3].baseboardY = OR.baseboardY;
    RC[3].x = OR.x + cranebase_reacharea_x - cranebase_origin_inset_x;
    RC[3].y = OR.y - cranebase_origin_inset_y;
    RC[3].z = OR.z;
    //-------------------------------------------------------------------------
    return RC;
  }
  // --------------------------------------------------------------------------
  // The two point straight line formulas for x and y for the points A and B:
  //       (by-ay)                        (bx-ax)
  //   y = ------- * (x-ax) + by      x = ------- * (y-ay) + bx
  //       (bx-ax)                        (by-ay)
  //---------------------------------------------------------------------------
  public float ValueXStraightline(
                 WorldCoordinate P, WorldCoordinate A, WorldCoordinate B)
  { if (A.y==B.y) return A.y;
    else return cast<float>((B.x-A.x)*(P.y-A.y)/(B.y-A.y)); 
    return 0.0;
  }
  public float ValueYStraightline(
                 WorldCoordinate P, WorldCoordinate A, WorldCoordinate B)
  { if (A.x==B.x) return A.x;
    else return cast<float>((B.y-A.y)*(P.x-A.x)/(B.x-A.x));
    return 0.0;
  }
  // --------------------------------------------------------------------------
  // InsideReacharea returns true if the point is reachable, otherwise false.
  // Edges of the reacharea rectangle are assumed outside.
  //---------------------------------------------------------------------------
  public bool InsideReacharea(WorldCoordinate P, WorldCoordinate[] W)
  {
    //-------------------------------------------------------------------------
    // Adjust the point to the same baseboard as the container crane
    //-------------------------------------------------------------------------
    WorldCoordinate SP = actsTools.WCoordAdjustBaseboard(P,me.GetMapObjectPosition());
    //-------------------------------------------------------------------------
    // Calculate the case to proof
    //-------------------------------------------------------------------------
    float rz = me.GetMapObjectOrientation().rz;
    int wi = cast<int>(rz*180*100/Math.PI);     // int rounded to 1/100
    int ri = wi/(90*100);
    while (ri<0) ri=ri+4;
    while (ri>3) ri=ri-4;
    //-------------------------------------------------------------------------
    // Values of the edges (straight ölines) for the point coordinates
    //-------------------------------------------------------------------------
    float xAB = ValueXStraightline(P, W[0], W[1]);
    float yAB = ValueYStraightline(P, W[0], W[1]);
    float xBC = ValueXStraightline(P, W[1], W[2]);
    float yBC = ValueYStraightline(P, W[1], W[2]);
    float xCD = ValueXStraightline(P, W[2], W[3]);
    float yCD = ValueYStraightline(P, W[2], W[3]);
    float xAD = ValueXStraightline(P, W[0], W[3]);
    float yAD = ValueYStraightline(P, W[0], W[3]);
    //-------------------------------------------------------------------------
    // Calculating the resulting bool value
    //-------------------------------------------------------------------------
    switch(ri)
    {
      case 0:  //  0° <= rz° <  90°
        if(yCD < P.y and P.y < yAB and xAD < P.x and P.x < xBC) return true;
        return false;
      case 1:  //  90° <= rz° < 180°
        if(yAD < P.y and P.y < yBC and xAB < P.x and P.x < xCD) return true;
        return false;
      case 2:  // 180° <= rz° < 270°
        if(yAB < P.y and P.y < yCD and xBC < P.x and P.x < xAD) return true;
        return false;
      case 3:  // 270° <= rz° < 360°
        if(yBC < P.y and P.y < yAD and xCD < P.x and P.x < xAB) return true;
        return false;
      default:
        return false;
    }
    return false;
  }
  //---------------------------------------------------------------------------
  
};
