include "acts_container_globals.gs"
include "acts_tools_script.gs"

//-----------------------------------------------------------------------------
// aCTS Container V0.1 Central Script - acts_container_script.gs
//-----------------------------------------------------------------------------
//   acts_container             <-- acts_container_script
// X acts_container_script      <-- acts_container_skinreplace
// X acts_container_skinreplace <-- acts_container_register
// X acts_container_register    <-- acts_container_globals
//   acts_container_globals     <-- acts_global_values
//   acts_global_values         <-- MapObject
//-----------------------------------------------------------------------------

class acts_container_register isclass acts_container_globals
{
  //---------------------------------------------------------------------------
  // TBD: Outsource from acts_container_script
  //---------------------------------------------------------------------------
};

class acts_container_skinreplace isclass acts_container_register
{
  //---------------------------------------------------------------------------
  // TBD: Outsource from acts_container_script
  //---------------------------------------------------------------------------
};

class acts_container_script isclass acts_container_skinreplace
{
  //---------------------------------------------------------------------------
  public void SetProperties(Soup soup)   // Callback method after Init method
  {
    inherited(soup);
    //-------------------------------------------------------------------------
    SkinDataSoup.Copy(soup.GetNamedSoup("SkinData"));
    ObjectDataSoup.Copy(soup.GetNamedSoup("ObjectData"));
    LocationSoup.Copy(soup.GetNamedSoup("Location"));
    OrientationSoup.Copy(soup.GetNamedSoup("Orientation"));
    ContentDataSoup.Copy(soup.GetNamedSoup("ContentData"));
    //-------------------------------------------------------------------------
    isSetPropertiesMethodFinnished = true;
    PostMessage(me,"OBJECTINIT","SETPROPS_FINNISHED",0.0);
    return;
  }
  //---------------------------------------------------------------------------
  public Soup GetProperties(void)   // Callback method before closing session 
  {
    Soup soup = inherited();
    //-------------------------------------------------------------------------
    if (acts_debug)Interface.Log(Interface.GetTimeStamp()
              + " [" + objectgoid.SerialiseToString() + "]"
              + " CONTAINER SKINDATASOUP");
    if (acts_debug)actsTools.ShowSoup(SkinDataSoup,"=>");
    //-------------------------------------------------------------------------*/
    soup.SetNamedSoup("SkinData",SkinDataSoup);
    soup.SetNamedSoup("ObjectData",ObjectDataSoup);
    soup.SetNamedSoup("Location",LocationSoup);
    soup.SetNamedSoup("Orientation",OrientationSoup);
    soup.SetNamedSoup("ContentData",ContentDataSoup);
    //-------------------------------------------------------------------------
    return soup;
  }
  //---------------------------------------------------------------------------
  public void WorldInitHandler(Message msg)
  {
    if (acts_debug)Interface.Log(Interface.GetTimeStamp()
            + " [" + objectgoid.SerialiseToString() + "]"
            + " CONTAINER GOT WORLDINIT MESSAGE ["
            + msg.major+"] ["+msg.minor+"]");
    //-------------------------------------------------------------------------*/
    // If one of the two methods (Init or SetProperties) isn't finished yet, 
    // post "World","ModuleInit" message a little bit later again.
    // So it is guaranteed that the Init and SetProperties methods are finished
    // as well as the "World","ModuleInit" message tells to be save using the 
    // World.GetCurrentModule method.
    //-------------------------------------------------------------------------
    if (!(isSetPropertiesMethodFinnished and isInitMethodFinnished))
    {
      PostMessage(me, "World", "ModuleInit", 0.1);
    }
    //-------------------------------------------------------------------------
    isWorldInitFinnished = true;
    PostMessage(me, "OBJECTINIT", "WORLD_INIT_FINNISHED", 0.0);
    //-------------------------------------------------------------------------
    return;
  }
  //---------------------------------------------------------------------------
  public void ObjectInitHandler(Message msg)
  {
    //-------------------------------------------------------------------------
    // Until the Trainz core send the "World","ModuleInit" was send it is not
    // guarented to decide with the World.GetCurrentModule method to be in the 
    // returned module! 
    //-------------------------------------------------------------------------
    if (acts_debug)Interface.Log(Interface.GetTimeStamp()
            + " [" + objectgoid.SerialiseToString() + "]"
            + " CONTAINER GOT MESSAGE ["+msg.major+"] ["+msg.minor+"]");
    //-------------------------------------------------------------------------*/
    if (msg.dst==me and msg.major=="OBJECTINIT")
    {
      if (msg.minor=="INIT_FINNISHED")
      {
        //---------------------------------------------------------------------
        // Show the goid on both sides of me (for debugging purpose)
        //---------------------------------------------------------------------
        SetFXNameText("GOIDV","["+objectgoid.SerialiseToString()+"]");
        SetFXNameText("GOIDH","["+objectgoid.SerialiseToString()+"]");
        //---------------------------------------------------------------------
        skins = GetAsset().FindAsset("skinskuid");
        //---------------------------------------------------------------------
      }
      //-----------------------------------------------------------------------
      if (msg.minor=="SETPROPS_FINNISHED")
      {
        //---------------------------------------------------------------------
        // Adjust position and rotation to rounded values 
        // (see acts_toolsscript.gs)
        //---------------------------------------------------------------------
        //Soup sp = actsTools.AutoAdjustMapObject(me);
        //LocationSoup    = sp.GetNamedSoup("AdjustedLocationSoup");
        //OrientationSoup = sp.GetNamedSoup("AdjustedOrientationSoup");
        //---------------------------------------------------------------------
        // Random skin number or load skin number from soup
        // TBD: Additionally set the skin with the properties browser.
        //---------------------------------------------------------------------
        if (SkinDataSoup.CountTags()==0)
        {
          textcount = 9;
          skin = Math.Rand(1, textcount+1);
          skinDescription = "TR_Random_Texture [#"+skin+"]";
          SkinDataSoup.SetNamedTag("skin",skin);
          SkinDataSoup.SetNamedTag("skinDescription",skinDescription);
        }
        else
        {
          skin            = SkinDataSoup.GetNamedTagAsInt("skin");
          skinDescription = SkinDataSoup.GetNamedTag("skinDescription");
        }
        if (skin == 0) SetFXTextureReplacement("skincolor",null,0);
        else SetFXTextureReplacement("skincolor",skins,skin-1);
        //---------------------------------------------------------------------
        if (acts_debug)Interface.Log(Interface.GetTimeStamp()
                  + " [" + objectgoid.SerialiseToString() + "]"
                  + " CONTAINER SKIN ["+skin+"] ["+skinDescription+"]");
        if (acts_debug)actsTools.ShowSoup(SkinDataSoup,"=>");
        //---------------------------------------------------------------------*/
        if (ObjectDataSoup.CountTags()==0)
        {
          //-------------------------------------------------------------------
          // LoadObjectDataInfos(ObjectDataSoup);
          //-------------------------------------------------------------------
          ObjectDataSoup.SetNamedTag("goid",objectgoid);
          ObjectDataSoup.SetNamedTag("type",objecttype);
          ObjectDataSoup.SetNamedTag("length",contlength);
          ObjectDataSoup.SetNamedTag("height",contheight);
          ObjectDataSoup.SetNamedTag("regID",-1);
          ObjectDataSoup.SetNamedTag("active",false);   // False: Not reachable
          ObjectDataSoup.SetNamedTag("requested",false);
          ObjectDataSoup.SetNamedTag("hooked",false);
        }
        //---------------------------------------------------------------------
        if (acts_debug)Interface.Log(Interface.GetTimeStamp()
                    + " =>SOUP 'ObjectDataSoup'");
        if (acts_debug)actsTools.ShowSoup(ObjectDataSoup,"=>");
        //---------------------------------------------------------------------*/
        if (LocationSoup.CountTags()==0)
        {
          //-------------------------------------------------------------------
          // LoadLocationDataInfos(LocationDataSoup);
          //-------------------------------------------------------------------
          WorldCoordinate WC = me.GetMapObjectPosition();
          LocationSoup.SetNamedTag("bbx",WC.baseboardX);
          LocationSoup.SetNamedTag("bby",WC.baseboardY);
          LocationSoup.SetNamedTag("px",WC.x);
          LocationSoup.SetNamedTag("py",WC.y);
          LocationSoup.SetNamedTag("pz",WC.z);
        }
        //---------------------------------------------------------------------
        if (OrientationSoup.CountTags()==0)
        {
          //-------------------------------------------------------------------
          // LoadOrientationDataInfos(OrientationDataSoup);
          //-------------------------------------------------------------------
          Orientation OR = me.GetMapObjectOrientation();
          LocationSoup.SetNamedTag("rx",OR.rx);
          LocationSoup.SetNamedTag("ry",OR.ry);
          LocationSoup.SetNamedTag("rz",OR.rz);
          LocationSoup.SetNamedTag("gx",OR.rx*180.0/Math.PI);
          LocationSoup.SetNamedTag("gy",OR.ry*180.0/Math.PI);
          LocationSoup.SetNamedTag("gz",OR.rz*180.0/Math.PI);
        }
        //---------------------------------------------------------------------
      }
    }
  }
  //---------------------------------------------------------------------------
public void DispatcherHandler(Message msg)
  {
    if (msg.dst==me and msg.major=="DISPATCHER")
    {
      //-----------------------------------------------------------------------
      if (acts_debug)Interface.Log(Interface.GetTimeStamp()
              + " [" + objectgoid.SerialiseToString() + "] ["
              + " CONTAINER GOT MESSAGE FROM DISPATCHER"
              + (cast<GameObject>msg.src).GetGameObjectID().SerialiseToString()
              + "] ["+msg.major+"] ["+msg.minor+"]");
      //-----------------------------------------------------------------------*/
      if (msg.minor=="UPDATED")
      {
        //---------------------------------------------------------------------
        // Update the own ObjectDataSoup
        //---------------------------------------------------------------------
        ObjectDataSoup.Copy(cast<Soup>msg.paramSoup);
        if (acts_debug)Interface.Log(Interface.GetTimeStamp()
                + " SOUP CONTENT OF 'Updated DataSoup' GOID"
                + " [" + objectgoid.SerialiseToString() + "]");
        if (acts_debug)actsTools.ShowSoup(ObjectDataSoup,"=>");
        //---------------------------------------------------------------------*/
      }
      if (msg.minor=="INSERTED")
      {
        //---------------------------------------------------------------------
        // Update the own ObjectDataSoup
        //---------------------------------------------------------------------
        ObjectDataSoup.Copy(cast<Soup>msg.paramSoup);
        if (acts_debug)Interface.Log(Interface.GetTimeStamp()
                + " SOUP CONTENT OF 'Inserted DataSoup' GOID"
                + " [" + objectgoid.SerialiseToString() + "]");
        if (acts_debug)actsTools.ShowSoup(ObjectDataSoup,"=>");
        //---------------------------------------------------------------------*/
      }
      if (msg.minor=="APPENDED")
      {
        //---------------------------------------------------------------------
        // Update the own ObjectDataSoup
        //---------------------------------------------------------------------
        ObjectDataSoup.Copy(cast<Soup>msg.paramSoup);
        if (acts_debug)Interface.Log(Interface.GetTimeStamp()
                + " SOUP CONTENT OF 'Added DataSoup' GOID"
                + " [" + objectgoid.SerialiseToString() + "]");
        if (acts_debug)actsTools.ShowSoup(ObjectDataSoup,"=>");
        //---------------------------------------------------------------------*/
      }
    }
    //-------------------------------------------------------------------------
    return;
  }
  //---------------------------------------------------------------------------*/
};
