include "acts_contstack_globals.gs"
include "acts_tools_script.gs"

//-----------------------------------------------------------------------------
// aCTS Contstack V0.1 Central Script - acts_contstack_script.gs
//-----------------------------------------------------------------------------
//   acts_contstack             <-- acts_contstack_script
// X acts_contstack_script      <-- acts_contstack_register
// X acts_contstack_register    <-- acts_contstack_globals
//   acts_contstack_globals     <-- acts_global_values
//   acts_global_values         <-- MapObject
//-----------------------------------------------------------------------------

class acts_contstack_register isclass acts_contstack_globals
{
  //---------------------------------------------------------------------------
  // TBD: Outsource from acts_container_script
  //---------------------------------------------------------------------------
};

class acts_contstack_script isclass acts_contstack_register
{
  //---------------------------------------------------------------------------
  public void SetProperties(Soup soup)   // Callback method after Init method
  {
    inherited(soup);
    //-------------------------------------------------------------------------
    ObjectDataSoup.Copy(soup.GetNamedSoup("ObjectData"));
    LocationSoup.Copy(soup.GetNamedSoup("Location"));
    OrientationSoup.Copy(soup.GetNamedSoup("Orientation"));
    /*/-------------------------------------------------------------------------
    if (acts_debug) Interface.Log(Interface.GetTimeStamp()
            + " [" + objectgoid.SerialiseToString() + "]"
            + " CONTSTACK SETPROPERTIES SOUPS");
    if (acts_debug) actsTools.ShowSoup(ObjectDataSoup,"ObjDS=>");
    if (acts_debug) actsTools.ShowSoup(LocationSoup,"LocDS=>");
    if (acts_debug) actsTools.ShowSoup(OrientationSoup,"OriDS=>");
    //-------------------------------------------------------------------------*/
    isSetPropertiesMethodFinnished = true;
    PostMessage(me,"OBJECTINIT","SETPROPS_FINNISHED",0.0);
    return;
  }
  //---------------------------------------------------------------------------
  public Soup GetProperties(void)   // Callback method before closing session 
  {
    Soup soup = inherited();
    //-------------------------------------------------------------------------
    soup.SetNamedSoup("ObjectData",ObjectDataSoup);
    soup.SetNamedSoup("Location",LocationSoup);
    soup.SetNamedSoup("Orientation",OrientationSoup);
    //-------------------------------------------------------------------------
    if (acts_debug) Interface.Log(Interface.GetTimeStamp()
            + " [" + objectgoid.SerialiseToString() + "]"
            + " CONTSTACK GETPROPERTIES SOUPS");
    if (acts_debug) actsTools.ShowSoup(ObjectDataSoup,"ObjDS=>");
    if (acts_debug) actsTools.ShowSoup(LocationSoup,"LocDS=>");
    if (acts_debug) actsTools.ShowSoup(OrientationSoup,"OriDS=>");
    //-------------------------------------------------------------------------*/
    return soup;
  } 
  //---------------------------------------------------------------------------
  public void WorldInitHandler(Message msg)
  {
    //-------------------------------------------------------------------------
    if (acts_debug)Interface.Log(Interface.GetTimeStamp()
            + " [" + objectgoid.SerialiseToString() + "]"
            + " CONTSTACK GOT WORLDINIT MESSAGE ["
            + msg.major+"] ["+msg.minor+"]");
    //-------------------------------------------------------------------------*/
    // If one of the two methods (Init or SetProperties) isn't finished yet, 
    // post "World","ModuleInit" message a little bit later again.
    // So it is guaranteed that the Init and SetProperties methods are finished
    // as well as the "World","ModuleInit" message tells to be save using the 
    // World.GetCurrentModule method.
    //-------------------------------------------------------------------------
    if (!(isSetPropertiesMethodFinnished and isInitMethodFinnished))
    {
      PostMessage(me, "World", "ModuleInit", 0.1);
    }
    //-------------------------------------------------------------------------
    isWorldInitFinnished = true;
    PostMessage(me, "OBJECTINIT", "WORLD_INIT_FINNISHED", 0.0);
    //-------------------------------------------------------------------------
    return;
  }
  //---------------------------------------------------------------------------
  public void ObjectInitHandler(Message msg)
  {
    //-------------------------------------------------------------------------
    // Until the Trainz core send the "World","ModuleInit" was send it is not
    // guarented to decide with the World.GetCurrentModule method to be in the 
    // returned module! 
    //-------------------------------------------------------------------------
    if (acts_debug)Interface.Log(Interface.GetTimeStamp()
            + " [" + objectgoid.SerialiseToString() + "]"
            + " CONTSTACK GOT MESSAGE ["+msg.major+"] ["+msg.minor+"]");
    //-------------------------------------------------------------------------*/
    if (msg.dst==me and msg.major=="OBJECTINIT")
    {
      if (msg.minor=="INIT_FINNISHED")
      {
        //---------------------------------------------------------------------
        // Show the goid on both sides of me (for debugging purpose)
        //---------------------------------------------------------------------
        SetFXNameText("GOID","["+objectgoid.SerialiseToString()+"]");
        //---------------------------------------------------------------------
      }
      //-----------------------------------------------------------------------
      if (msg.minor=="SETPROPS_FINNISHED")
      {
        if (ObjectDataSoup.CountTags()==0)
        {
          //-------------------------------------------------------------------
          // LoadRegisterDataInfos(RegisterDataSoup);
          //-------------------------------------------------------------------
          ObjectDataSoup.SetNamedTag("goid",objectgoid);
          ObjectDataSoup.SetNamedTag("type","CONTSTACK");
          ObjectDataSoup.SetNamedTag("length",stacklength);
          ObjectDataSoup.SetNamedTag("height",stackheight);
          ObjectDataSoup.SetNamedTag("regID",-1);
          ObjectDataSoup.SetNamedTag("active",false);   // False: Not reachable
          ObjectDataSoup.SetNamedTag("requested",false);
          ObjectDataSoup.SetNamedTag("blocked",false);
        }
        //---------------------------------------------------------------------
        if (acts_debug)Interface.Log(Interface.GetTimeStamp()
                    + " =>SOUP 'ObjectDataSoup'");
        if (acts_debug)actsTools.ShowSoup(ObjectDataSoup,"=>");
        //---------------------------------------------------------------------*/
        if (LocationSoup.CountTags()==0)
        {
          //-------------------------------------------------------------------
          // LoadLocationDataInfos(LocationDataSoup);
          //-------------------------------------------------------------------
          WorldCoordinate WC = me.GetMapObjectPosition();
          LocationSoup.SetNamedTag("bbx",WC.baseboardX);
          LocationSoup.SetNamedTag("bby",WC.baseboardY);
          LocationSoup.SetNamedTag("px",WC.x);
          LocationSoup.SetNamedTag("py",WC.y);
          LocationSoup.SetNamedTag("pz",WC.z);
        }
        //---------------------------------------------------------------------
        if (OrientationSoup.CountTags()==0)
        {
          //-------------------------------------------------------------------
          // LoadOrientationDataInfos(OrientationDataSoup);
          //-------------------------------------------------------------------
          Orientation OR = me.GetMapObjectOrientation();
          LocationSoup.SetNamedTag("rx",OR.rx);
          LocationSoup.SetNamedTag("ry",OR.ry);
          LocationSoup.SetNamedTag("rz",OR.rz);
          LocationSoup.SetNamedTag("gx",OR.rx*180.0/Math.PI);
          LocationSoup.SetNamedTag("gy",OR.ry*180.0/Math.PI);
          LocationSoup.SetNamedTag("gz",OR.rz*180.0/Math.PI);
        }
        //---------------------------------------------------------------------
        // TBD: Adjust position and rotation to rounded values and handle
        //      the different pssibilities.
        //      (some methods see acts_tools_script.gs)
        //---------------------------------------------------------------------
        // Soup sp = actsTools.AutoAdjustMapObject(me);
        // LocationSoup    = sp.GetNamedSoup("AdjustedLocationSoup");
        // OrientationSoup = sp.GetNamedSoup("AdjustedOrientationSoup");
        //---------------------------------------------------------------------
      }
    }
  }
  //---------------------------------------------------------------------------
  public void DispatcherHandler(Message msg)
  {
    if (msg.dst==me and msg.major=="DISPATCHER")
    {
      //-----------------------------------------------------------------------
      if (acts_debug)Interface.Log(Interface.GetTimeStamp()
              + " ["+objectgoid.SerialiseToString()+"]"
              + " CONTSTACK GOT MESSAGE FROM DISPATCHER"
              + (cast<GameObject>msg.src).GetGameObjectID().SerialiseToString()
              + "] ["+msg.major+"] ["+msg.minor+"]");
      //-----------------------------------------------------------------------*/
      if (msg.minor=="OBJECTSOUP_UPDATED")
      {
        //---------------------------------------------------------------------
        // Update the own ObjectDataSoup
        //---------------------------------------------------------------------
        ObjectDataSoup.Copy(cast<Soup>msg.paramSoup);
        //---------------------------------------------------------------------*/
      }
      if (msg.minor=="OBJECTSOUP_INSERTED")
      {
        //---------------------------------------------------------------------
        // Update the own ObjectDataSoup
        //---------------------------------------------------------------------
        ObjectDataSoup.Copy(cast<Soup>msg.paramSoup);
        //---------------------------------------------------------------------*/
      }
      if (msg.minor=="OBJECTSOUP_APPENDED")
      {
        //---------------------------------------------------------------------
        // Update the own ObjectDataSoup
        //---------------------------------------------------------------------
        ObjectDataSoup.Copy(cast<Soup>msg.paramSoup);
        //---------------------------------------------------------------------*/
      }
    }
    //-------------------------------------------------------------------------
    return;
  }
};
