include "acts_global_values.gs"
include "acts_tools_script.gs"

//-----------------------------------------------------------------------------
// aCTS Dispatcher V0.1 Central Script - acts_dispatcher
//-----------------------------------------------------------------------------
// X acts_dispatcher             <-- acts_dispatcher_script
// X acts_dispatcher_script      <-- acts_dispatcher_globals
// X acts_dispatcher_globals     <-- acts_global_values
//   acts_global_values          <-- MapObject
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
class acts_dispatcher_globals isclass acts_global_values
{
  //---------------------------------------------------------------------------
  // Store from dispatcher received object registration values to the 
  // respective dispatchers sub soup. The sub soups register ids run as int  
  // from 0 ... The received id -1 tells, that the object isn't yet
  // registered.
  //---------------------------------------------------------------------------
  // Because the register id number list gets gaps while unregistering it will
  // be nessecary to have some gap filling methods additionally. See
  // AppendInsertUpdateObjectDataSoups and FindObjectSoupEntry methods.
  //---------------------------------------------------------------------------
  // AssetInstanceSoup {
  //   ObjectDataSoup { ... },    Only container, contstack, contcrane
  //   LocationSoup { ... },      Only container, contstack, contcrane
  //   OrientationSoup { ... },   Only container, contstack, contcrane
  //   SkinDataSoup { ... },      Only container
  //   ContentDataSoup { ... },   Only container
  // X DispatcherDataSoup { ... } Only dispatcher
  // X ContainerSoups  { ... }    Only dispatcher
  // X ContstackSoups  { ... }    Only dispatcher
  // X ContcraneSoups  { ... }    Only dispatcher
  // }
  //---------------------------------------------------------------------------
  Soup ContainerSoups = Constructors.NewSoup();
  Soup ContstackSoups = Constructors.NewSoup();
  Soup ContcraneSoups = Constructors.NewSoup();
  //---------------------------------------------------------------------------
  Soup DispatcherDataSoup = Constructors.NewSoup();
  //---------------------------------------------------------------------------
  // ObjectDataSoup received from and posted to objects.
  //---------------------------------------------------------------------------
  // ObjectDataSoup { goid GameObjectID, type string, dispID int,
  //                  length int, height int,
  //                  stack goid, place int,           Only container
  //                  active bool, action string },
  // activ: true means the object sended a register message while session run.
  //        All objects were set false while initialising dispatcher and if
  //        they stay false their entries will be deleted next initialising.
  // actions: ["APPENDED", "INSERTED", "UPDATED"];
  //---------------------------------------------------------------------------
  GameObjectID dispgoid = me.GetGameObjectID();
  string objecttype;
  GameObjectID objectgoid;
  //---------------------------------------------------------------------------
};

class acts_dispatcher_script isclass acts_dispatcher_globals
{
  //---------------------------------------------------------------------------
  public void SetProperties(Soup soup)   // Callback method after Init method
  {
    inherited(soup);
    //-------------------------------------------------------------------------
    DispatcherDataSoup.Copy(soup.GetNamedSoup("DispatcherData"));
    ContainerSoups.Copy(soup.GetNamedSoup("ContainerSoups"));
    ContstackSoups.Copy(soup.GetNamedSoup("ContstackSoups"));
    ContcraneSoups.Copy(soup.GetNamedSoup("ContcraneSoups"));
    //-------------------------------------------------------------------------
    isSetPropertiesMethodFinnished = true;
    PostMessage(me,"OBJECTINIT","SETPROPS_FINNISH",0);
    return;
  }
  //---------------------------------------------------------------------------
  public Soup GetProperties(void)   // Callback method before closing session 
  {
    Soup soup = inherited();
    //-------------------------------------------------------------------------
    DispatcherDataSoup.SetNamedTag("firstrun",false);
    //-------------------------------------------------------------------------
    soup.SetNamedSoup("DispatcherData",DispatcherDataSoup);
    soup.SetNamedSoup("ContainerSoups",ContainerSoups);
    soup.SetNamedSoup("ContstackSoups",ContstackSoups);
    soup.SetNamedSoup("ContcraneSoups",ContcraneSoups);
    //-------------------------------------------------------------------------
    return soup;
  } 
  //---------------------------------------------------------------------------
  public void WorldInitHandler(Message msg)
  {
    if (acts_debug) Interface.Log(Interface.GetTimeStamp()
            + " [" + dispgoid.SerialiseToString() + "]"
            + " DISPATCHER GOT WORLDINIT MESSAGE ["
            + msg.major+"] ["+msg.minor+"]");
    //-------------------------------------------------------------------------
    // If one of the two methods (Init or SetProperties) isn't finished yet, 
    // post "World","ModuleInit" message a little bit later again.
    // So it is guaranteed that the Init and SetProperties methods are finished
    // as well as the "World","ModuleInit" message tells to be save using the 
    // World.GetCurrentModule method.
    //-------------------------------------------------------------------------
    if (!(isSetPropertiesMethodFinnished and isInitMethodFinnished))
    {
       //----------------------------------------------------------------------
       // While first run and before first session save for libraries the 
       // SetProperties methon will not run, whle no soup stored. If this case
       // isn't handeled, the repost of "World","ModuleInit" runs infinitly.
       //----------------------------------------------------------------------
       if (!DispatcherDataSoup.GetNamedTagAsBool("firstrun")) 
          PostMessage(me, "World", "ModuleInit", 0.1);
    }
    //-------------------------------------------------------------------------
    isWorldInitFinnished = true;
    PostMessage(me, "OBJECTINIT", "WORLD_INIT_FINNISHED", 0.0);
    //-------------------------------------------------------------------------
    return;
  }
  //---------------------------------------------------------------------------
  public void ObjectInitHandler(Message msg)
  {
    //-------------------------------------------------------------------------
    // Until the Trainz core send the "World","ModuleInit" message it is not
    // guarented to decide with the World.GetCurrentModule method to be in the 
    // returned module! 
    //-------------------------------------------------------------------------
    if (acts_debug) Interface.Log(Interface.GetTimeStamp()
            + " [" + dispgoid.SerialiseToString() + "]"
            + " DISPATCHER GOT MESSAGE ["+msg.major+"] ["+msg.minor+"]");
    //-------------------------------------------------------------------------*/
    if (msg.dst==me and msg.major=="OBJECTINIT")
    {
      //-----------------------------------------------------------------------
      if (msg.minor=="INIT_FINNISHED")
      {
        //---------------------------------------------------------------------
        // Not used yet
        //---------------------------------------------------------------------
      }
      //-----------------------------------------------------------------------
      if (msg.minor=="SETPROPS_FINNISHED")
      {
        //---------------------------------------------------------------------
        // If DispatcherDataSoup is empty, this is the first call unsaved call
        //---------------------------------------------------------------------
        if (DispatcherDataSoup.CountTags()==0)
           DispatcherDataSoup.SetNamedTag("firstrun",true);
      }
      //-----------------------------------------------------------------------
      if (msg.minor=="WORLD_INIT_FINNISHED")
      {
        //---------------------------------------------------------------------
        // Do no module only, driver module only or surveyor module only
        // related initiations
        //---------------------------------------------------------------------
        if (World.GetCurrentModule() == World.DRIVER_MODULE)
        {
          //-------------------------------------------------------------------
          // Not used yet
          //-------------------------------------------------------------------
        }
        //---------------------------------------------------------------------
        if (World.GetCurrentModule() == World.SURVEYOR_MODULE)
        {
          //-------------------------------------------------------------------
          // Not used yet
          //-------------------------------------------------------------------
        }
        //---------------------------------------------------------------------
        if (World.GetCurrentModule() == World.NO_MODULE)
        {
          //-------------------------------------------------------------------
          // Not used yet
          //-------------------------------------------------------------------
        }
      }
    }
  }
  //---------------------------------------------------------------------------
  // Search the objects soup for next free entry gap. Parameter dsp is the
  // object's type dispatcher sub soup (ContainerSoups, ContstackSoups or
  // ContcraneSoups). Method returns a soup.
  // soup { entry int, action string, active bool }
  //---------------------------------------------------------------------------
  Soup FindObjectSoupEntry(GameObjectID goid, Soup dsp)
  {
    int entry;
    string action;
    Soup rsp = Constructors.NewSoup();
    rsp.SetNamedTag("active",true);
    int i;
    for (i=0; i<dsp.CountTags(); i++)
    {
      entry = Str.ToInt(dsp.GetIndexedTagName(i));
      Soup sp = dsp.GetNamedSoup(entry);
      if (sp.GetNamedTagAsGameObjectID("goid")==goid)   // TBD: goid compar ison
      {
        rsp.SetNamedTag("entry",entry);
        rsp.SetNamedTag("action","UPDATED");
        return rsp;
      }
    }
    entry = actsTools.GetObjectSoupGap(dsp);
    if (entry==dsp.CountTags()) rsp.SetNamedTag("action","APPENDED");
    else rsp.SetNamedTag("action","INSERTED");
    rsp.SetNamedTag("entry",entry);
    return rsp;
  }
  //---------------------------------------------------------------------------
  // Search the goid and update, insert or append the soup to register
  // Parameter osp is the posted ObjectDataSoup. The other three are the
  // dispatcher's object soups subsoups.
  //---------------------------------------------------------------------------
  void AppendInsertUpdateObjectDataSoups(Soup osp,
                    Soup ContSps, Soup StackSps, Soup CraneSps)
  { 
    //-------------------------------------------------------------------------
    string type = osp.GetNamedTag("type");
    GameObjectID objgoid = osp.GetNamedTagAsGameObjectID("type");
    Soup objsps = Constructors.NewSoup();
    if (type == "CONTAINER") objsps = ContSps;
    if (type == "CONTSTACK") objsps = StackSps;
    if (type == "CONTCRANE") objsps = CraneSps;
    Soup entry = FindObjectSoupEntry(objgoid, objsps);
    // entry { entry int, active bool, action string }
    // actions = ["APPENDED", "INSERTED", "UPDATED"];
    osp.SetNamedTag("active", true);
    osp.SetNamedTag("action", entry.GetNamedTag("action"));
    objsps.SetNamedSoup(entry.GetNamedTagAsInt("entry"), osp);
    //-------------------------------------------------------------------------
    // Post retour that register is updated
    //-------------------------------------------------------------------------
    GameObject obj = Router.GetGameObject(objgoid);
    PostMessage(obj, "DISPATCHER",
                     "OBJECTSOUP_" + entry.GetNamedTag("action"), osp, 0.0);
    //-------------------------------------------------------------------------
    if (acts_debug) Interface.Log(Interface.GetTimeStamp()
            + " [" + dispgoid.SerialiseToString() + "]"
            + " DISPATCHER OBJECTSOUP " + entry.GetNamedTag("action") 
            + " " + type + " GOID [" + objgoid.SerialiseToString() + "]");
    if (acts_debug) Interface.Log(Interface.GetTimeStamp()
            + " CONTENT OF " + entry.GetNamedTag("action") + " SOUP FROM GOID"
            + " [" + objgoid.SerialiseToString() + "]");
    if (acts_debug) actsTools.ShowSoup(osp,"OSP=>");   
    //-------------------------------------------------------------------------*/
    return;
  }
  //---------------------------------------------------------------------------
  public void RegisterHandler(Message msg)
  {
    if (msg.dst==me and msg.major=="REGISTER")
    {
      //-----------------------------------------------------------------------
      if (acts_debug) Interface.Log(Interface.GetTimeStamp()
              + " ["+dispgoid.SerialiseToString()+"]"
              + " DISPATCHER GOT MESSAGE FROM OBJECT ["
              + (cast<GameObject>msg.src).GetGameObjectID().SerialiseToString()
              + "] [" + msg.major + "] [" + msg.minor + "]");
      //-----------------------------------------------------------------------*/
      if (msg.minor=="REGISTER_ME")
      {
        //---------------------------------------------------------------------
        // Update, insert or append the ObjectDataSoup
        //---------------------------------------------------------------------
        Soup RegDataSoup = Constructors.NewSoup();
        RegDataSoup.Copy(cast<Soup>msg.paramSoup);
        //---------------------------------------------------------------------
        if (acts_debug) Interface.Log(Interface.GetTimeStamp()
                      + " SOUP CONTENT OF 'DataSoup To Register' GOID"
                      + " [" + dispgoid.SerialiseToString() + "]");
        if (acts_debug) actsTools.ShowSoup(RegDataSoup,"=>");
        //---------------------------------------------------------------------
        AppendInsertUpdateObjectDataSoups(RegDataSoup, 
                          ContainerSoups, ContstackSoups, ContcraneSoups);
        //---------------------------------------------------------------------
        // TBD: Repost or update additional informations
        //   * CONTCRANE: Repost a goid list of reachable stacks. The contcrane
        //       itself informs the stacks about it's object data to update the
        //       maximum count of stackable containers.
        //   * CONTSTACK: ???
        //   * CONTAINER: If a container has no stack place, randomly attach
        //       him to a free stack place and repost it. In case there is no
        //       one, repost an empty stack goid with place zero.
        //       TBD: What happens if the stack it attached to does not exist?
        //            Possibly handled as a free container?       
        //---------------------------------------------------------------------
      }
      //-----------------------------------------------------------------------
    }
    return;
  }
};

class acts_dispatcher isclass acts_dispatcher_script
{
  public void Init()
  {
    inherited();
    //-------------------------------------------------------------------------
    // Make shure that the right module is set for World.GetCurrentModle()
    // (see world.gs lines 939ff.) to start driver only or surveyor only
    // related initiations and tell me, if my Init and SetProperties methods
    // are finished
    //-------------------------------------------------------------------------
    AddHandler(me, "World", "ModuleInit", "WorldInitHandler");
    AddHandler(me, "OBJECTINIT", "", "ObjectInitHandler");
    AddHandler(me, "REGISTER",   "", "RegisterHandler");
    //-------------------------------------------------------------------------
    // Set asset special initial values
    //-------------------------------------------------------------------------
    objecttype = "DISPATCHER";
    //-------------------------------------------------------------------------
    isInitMethodFinnished = true;
    PostMessage(me,"OBJECTINIT","INIT_FINNISHED",0.0);   // Init method finnished
    return;
  }
};
	