include "acts_global_values.gs"

//-----------------------------------------------------------------------------
// Common aCTS methods V0.1 - Dispatcher, Container, Stack, Crane Tools Script
//-----------------------------------------------------------------------------

final static class actsTools isclass acts_global_values
{
  //---------------------------------------------------------------------------
  // Log soup content of a soup of soups only and without subsoups
  //---------------------------------------------------------------------------
  public void ShowSoup(Soup sp, string ind)
  {
    if (sp.CountTags()!=0)
    {
      int i;
      for (i=0; i<sp.CountTags(); i++)
        if (acts_debug) Interface.Log(ind + sp.GetIndexedTagName(i)
                + " ["+sp.GetNamedTag(sp.GetIndexedTagName(i))+"]");
    }
    return;
  }
  public void ShowSoups(Soup soup, string indent)
  {
    string ind = "="+indent;
    int sc = soup.CountTags();
    int i;
    for (i=0; i<sc; i++)
    {
      if (acts_debug) Interface.Log(indent + i + ".SubSoup");
      actsTools.ShowSoup(soup.GetNamedSoup(i),ind);
    }
    return;
  }
  //---------------------------------------------------------------------------
  // Set all ObjectSoups active false and get the last insert gap
  // TBD: Starting with a sorted soup 
  //---------------------------------------------------------------------------
  public void ResetObjectSoups(Soup soup)
  {
    int i;
    int gap = soup.CountTags();;
    for (i=0; i<soup.CountTags(); i++)
    {
      int tn = Str.ToInt(soup.GetIndexedTagName(i));
      Soup sp = soup.GetNamedSoup(tn);
      if (sp.GetNamedTagAsBool("active"))
      {
        sp.SetNamedTag("active",false);
        soup.SetNamedSoup(tn,sp);
      } else soup.RemoveNamedTag(tn); 
    }
    return;
  }
  //---------------------------------------------------------------------------
  // Return the first indexed gap in existing objectsoups tagnames
  //---------------------------------------------------------------------------
  public int GetObjectSoupGap(Soup soup)
  {
    int i;
    for (i=0; i<soup.CountTags(); i++)
      if (soup.GetIndexForNamedTag(i)==-1) return i;
    return soup.CountTags();
  }
  //---------------------------------------------------------------------------
  // Generate soup representation from WorldCoordinate
  //---------------------------------------------------------------------------
  public Soup WCoordToSoup(WorldCoordinate pos)
  {
    Soup sp = Constructors.NewSoup();
    sp.SetNamedTag("bbx",pos.baseboardX);
    sp.SetNamedTag("bby",pos.baseboardY);
    sp.SetNamedTag("px",pos.x);
    sp.SetNamedTag("py",pos.y);
    sp.SetNamedTag("pz",pos.z);
    return sp;
  }
  //---------------------------------------------------------------------------
  // Generate WorldCoordinate from its soup representation
  //---------------------------------------------------------------------------
  public WorldCoordinate WCoordFromSoup(Soup sp)
  {
    WorldCoordinate WC;
    WC.baseboardX = sp.GetNamedTagAsInt("bbx");
    WC.baseboardY = sp.GetNamedTagAsInt("bby");
    WC.x = sp.GetNamedTagAsFloat("px");
    WC.y = sp.GetNamedTagAsFloat("py");
    WC.z = sp.GetNamedTagAsFloat("pz");
    return WC;
  }
  //---------------------------------------------------------------------------
  // Normalize the WC to baseboard width
  //---------------------------------------------------------------------------
  public WorldCoordinate WCoordNormalize(WorldCoordinate WC)
  {
    WorldCoordinate nWC;
    //-------------------------------------------------------------------------
    int bx = WC.x * 1000.0 / baseboardwidth;   // Basebord delta
    if (WC.x < 0.0) bx--;
    nWC.baseboardX = WC.baseboardX + bx;
    nWC.x = WC.x - bx * baseboardwidth;
    //-------------------------------------------------------------------------
    int by = WC.y * 1000.0 / baseboardwidth;   // Basebord delta
    if (WC.y < 0.0) by--;
    nWC.baseboardY = WC.baseboardX + by;
    nWC.y = WC.y - by * baseboardwidth;
    //-------------------------------------------------------------------------
    nWC.z = WC.z;
    return nWC;
  }
  //---------------------------------------------------------------------------
  // Adjust baseboard from one WorlCoordinate to another
  //---------------------------------------------------------------------------
  public WorldCoordinate 
         WCoordAdjustBaseboard(WorldCoordinate frWC, WorldCoordinate toWC)
  {
    WorldCoordinate adjWC;
    //-------------------------------------------------------------------------
    adjWC.x = toWC.x + (frWC.baseboardX - toWC.baseboardX)*baseboardwidth;
    adjWC.baseboardX = frWC.baseboardX;
    adjWC.y = toWC.y + (frWC.baseboardY - toWC.baseboardY)*baseboardwidth;
    adjWC.baseboardY = frWC.baseboardY;
    adjWC.z = toWC.z;
    //-------------------------------------------------------------------------
    return adjWC;
  }
  //---------------------------------------------------------------------------
  // Generate soup representation from Orientation
  //---------------------------------------------------------------------------
  public Soup OrientationToSoup(Orientation rot)
  {
    Soup sp = Constructors.NewSoup();
    sp.SetNamedTag("gx",rot.rx * 1800.0 / Math.PI);
    sp.SetNamedTag("gy",rot.ry * 1800.0 / Math.PI);
    sp.SetNamedTag("gz",rot.rz * 1800.0 / Math.PI);
    sp.SetNamedTag("rx",rot.rx);
    sp.SetNamedTag("ry",rot.ry);
    sp.SetNamedTag("rz",rot.rz);
    return sp;
  }
  //---------------------------------------------------------------------------
  // Generate Orientation from its soup representation
  //---------------------------------------------------------------------------
  public Orientation OrientationFromSoup(Soup sp)
  {
    Orientation OR;
    OR.rx = sp.GetNamedTagAsFloat("rx");
    OR.ry = sp.GetNamedTagAsFloat("ry");
    OR.rz = sp.GetNamedTagAsFloat("rz");
    return OR;
  }
  //---------------------------------------------------------------------------
  // Adjust location and orientation to even mm and even 1/10° inplace
  //---------------------------------------------------------------------------
  public bool AutoAdjustTransformation(WorldCoordinate W, Orientation O)
  {
    float rd;
    bool changed = false;
    if (W.x<0) rd=-0.0005; else rd=0.0005;   // rounding to full [mm]
    int wcx = (W.x+rd) * 1000.0;
    if (W.y<0) rd=-0.0005; else rd=0.0005;
    int wcy = (W.y+rd) * 1000.0;
    if (W.z<0) rd=-0.0005; else rd=0.0005;
    int wcz = (W.z+rd) * 1000.0;
    // rounding to full [1/10°] - here only z-axis, x,y-axis are set to zero
    // to make shure the CTS is horizontal plane
    if (O.rz<0) rd=-0.5; else rd=0.5;
    int orz = O.rz * 1800.0 / Math.PI + rd;
    if (W.x != wcx/1000.0) {W.x = wcx/1000.0; changed=true; }
    if (W.y != wcy/1000.0) {W.y = wcy/1000.0; changed=true; }
    if(W.z != 0.0) { W.z = 0.0; changed=true; }
    if(O.rx != 0.0) { O.rx = 0.0; changed=true; }
    if(O.ry != 0.0) { O.ry = 0.0; changed=true; }
    if(O.rz != orz * Math.PI / 1800.0) 
    { O.rz = orz * Math.PI / 1800.0; changed=true; }
    //-------------------------------------------------------------------------
    return changed;
  }
  //---------------------------------------------------------------------------
  // Auto adjust WorldCoordinate and Orientation to [mm] and [1/10°]
  // Adjusts the object and returns original and adjusted WC and OR as soup.
  //---------------------------------------------------------------------------
  public Soup AutoAdjustMapObject(MapObject M)
  {
    //-------------------------------------------------------------------------
    // M = cast<MapObject>(me);
    //-------------------------------------------------------------------------
    bool changed = false;    
    Soup sp = Constructors.NewSoup();
    WorldCoordinate W = M.GetMapObjectPosition();
    Orientation     O = M.GetMapObjectOrientation();
    sp.SetNamedSoup("OriginalLocationSoup", WCoordToSoup(W));
    sp.SetNamedSoup("OriginalOrientationSoup", OrientationToSoup(O));
    changed = AutoAdjustTransformation(W,O);
    M.SetMapObjectPosition(W);
    M.SetMapObjectOrientation(O);
    sp.SetNamedSoup("AdjustedLocationSoup", WCoordToSoup(W));
    sp.SetNamedSoup("AdjustedOrientationSoup", OrientationToSoup(O));
    sp.SetNamedTag("changed", changed);
    //-------------------------------------------------------------------------
    return sp;
  }
};
